import asyncio
from cpl_core.database.context.database_context import DatabaseContext
import discord
from discord.channel import VoiceChannel
from discord.ext import commands
from discord.ext.commands import Context

from cpl_core.logging import LoggerABC
from cpl_core.configuration import ConfigurationABC
from gismo_core.abc.bot_service_abc import BotServiceABC

from gismo_core.abc.command_abc import CommandABC
from gismo_core.abc.message_service_abc import MessageServiceABC
from gismo_core.configuration.server_settings import ServerSettings
from gismo_data.abc.client_repository_abc import ClientRepositoryABC
from modules.base.base_settings import BaseSettings
from modules.permission.abc.permission_service_abc import PermissionServiceABC


class BaseCommandService(CommandABC):

    def __init__(
        self,
        logger: LoggerABC,
        config: ConfigurationABC,
        message_service: MessageServiceABC,
        permissions: PermissionServiceABC,
        clients: ClientRepositoryABC,
        db: DatabaseContext,
        bot: BotServiceABC
    ):
        CommandABC.__init__(self)

        self._logger = logger
        self._config = config
        self._message_service = message_service
        self._permissions = permissions
        self._clients = clients
        self._db = db
        self._bot = bot

        self._logger.trace(__name__, f'Loaded command service: {type(self).__name__}')

    @commands.command()
    async def ping(self, ctx: Context):
        self._logger.debug(__name__, f'Received command ping {ctx}')
        await self._message_service.send_ctx_msg(ctx, 'Pong')
        self._logger.trace(__name__, f'Finished ping command')

    @commands.command()
    async def purge(self, ctx: Context):
        self._logger.debug(__name__, f'Received command purge {ctx}')
        settings: BaseSettings = self._config.get_configuration(f'Base_{ctx.guild.id}')
        server_settings: ServerSettings = self._config.get_configuration(f'DSERVER_{ctx.guild.id}')
        
        if not self._permissions.is_member_moderator(ctx.author):
            await self._message_service.send_ctx_msg(ctx, settings.no_permissions_message)
            self._logger.trace(__name__, f'Finished purge command')
            return

        await self._message_service.send_ctx_msg(ctx, settings.purge_message)
        await asyncio.sleep(server_settings.message_delete_timer)
        try:
            await ctx.channel.purge()
        except Exception as e:
            self._logger.error(__name__, f'Cannot purge channel {ctx.channel.id}', e)
            await self._message_service.send_ctx_msg(ctx, server_settings.bot_has_no_permission_message)

        self._logger.trace(__name__, f'Finished purge command')

    @commands.command()
    async def afk(self, ctx: Context):
        self._logger.debug(__name__, f'Received command afk {ctx}')
        settings: BaseSettings = self._config.get_configuration(f'Base_{ctx.guild.id}')
        server_settings: ServerSettings = self._config.get_configuration(f'DSERVER_{ctx.guild.id}')

        if ctx.author.voice is None or ctx.author.voice.channel is None:
            await self._message_service.send_ctx_msg(ctx, settings.afk_command_channel_missing_message)
            self._logger.trace(__name__, f'Finished afk command')
            return

        self._bot.loop.create_task(self._message_service.send_ctx_msg(ctx, settings.afk_command_move_message))
        channel: VoiceChannel = ctx.guild.get_channel(settings.afk_command_channel_id)
        try:
            await ctx.author.move_to(channel)

            client = self._clients.get_client_by_discord_id(self._bot.user.id)
            client.moved_users_count += 1
            self._clients.update_client(client)
            self._db.save_changes()
        except Exception as e:
            self._logger.error(__name__, f'Cannot move user {ctx.author.id} to channel {ctx.channel.id}', e)
            await self._message_service.send_ctx_msg(ctx, server_settings.bot_has_no_permission_message)
        
        self._logger.trace(__name__, f'Finished afk command')
